@@ -1,22 +1,7 @@
-{-# LANGUAGE NoImplicitPrelude #-}
-
 {-
     BNF Converter: Named instance variables
     Copyright (C) 2004  Author:  Michael Pellauer
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335, USA
 -}
 
 
@@ -63,19 +48,20 @@ struct A {
 @
 
 This is what this module does.
-
 -}
 
 module BNFC.Backend.Common.NamedVariables where
 
-import Prelude'
+import Control.Arrow (left, (&&&))
+import Data.Char     (toLower)
+import Data.Either   (lefts)
+import Data.List     (nub)
+import Data.Map      (Map)
+
+import Text.PrettyPrint (Doc)
+import qualified Text.PrettyPrint as P
 
 import BNFC.CF
-import Data.Char (toLower)
-import Data.List (nub)
-import Text.PrettyPrint
-import Control.Arrow (left, (&&&))
-import Data.Either (lefts)
 
 type IVar = (String, Int)
 --The type of an instance variable
@@ -83,9 +69,18 @@ type IVar = (String, Int)
 
 type UserDef = TokenCat --user-defined types
 
+-- | A symbol-mapping environment.
+type SymEnv = KeywordEnv
 
---A symbol-mapping environment.
-type SymEnv = [(String, String)]
+-- | Map keywords to their token name.
+type KeywordEnv = [(String, String)]
+
+-- | Map keywords and user-defined token types to their token name.
+type SymMap = Map SymKey String
+data SymKey
+  = Keyword String    -- ^ Keyword like "(", "while", "true", ...
+  | Tokentype String  -- ^ Token type like "Integer", "Char", ...
+  deriving (Eq, Ord, Show)
 
 -- | Converts a list of categories into their types to be used as instance
 -- variables. If a category appears only once, it is given the number 0,
@@ -95,7 +90,6 @@ type SymEnv = [(String, String)]
 -- [("A",1),("B",0),("A",2)]
 --
 getVars :: [Cat] -> [IVar]
-getVars [] = []
 getVars cs = foldl addVar [] (map identCat cs)
   where
     addVar vs = addVar' vs 0
@@ -131,41 +125,33 @@ numVars cats =
             let i = maybe 1 (+1) (lookup n env)
             -- Is there more use of the name u_ ?
                 thereIsMore = n `elem` map snd (lefts xs)
-                vname = text n <> if i > 1 || thereIsMore then int i else empty
+                vname = P.text n P.<> if i > 1 || thereIsMore then P.int i else P.empty
             in Left (c, vname) : f' ((n,i):env) xs
 
---This makes numbers a little nicer.
---If there's only one variable of a type we drop the redundant _1 label.
---(Actually here we add _1 labels to variables that need it, but the effect
--- is the same.)
-fixOnes :: Eq b => [Either String b] -> [Either String b]
-fixOnes [] = []
-fixOnes (Right f : fs) = Right f : fixOnes fs
-fixOnes (Left f : fs) =
-  if Left (f ++ "2") `elem` fs
-    then Left (f ++ "1") : fixOnes fs
-    else Left f : fixOnes fs
 
 --This fixes the problem with coercions.
 fixCoercions :: [(Cat, [Rule])] -> [(Cat, [Rule])]
 fixCoercions rs = nub (fixAll rs rs)
- where
+  where
   fixCoercion :: Cat -> [(Cat, [Rule])] -> [Rule]
   fixCoercion _ [] = []
   fixCoercion cat ((c,rules):cats) = if normCat c == normCat cat
     then rules ++ fixCoercion cat cats
     else fixCoercion cat cats
+
   fixAll :: [(Cat, [Rule])] -> [(Cat, [Rule])] -> [(Cat, [Rule])]
   fixAll _ [] = []
-  fixAll top ((cat,_):cats) = if isCoercion (show cat) -- This is weird: isCoercion is supposed to be applied to functions!!!!
+  fixAll top ((cat,_):cats) = if isCoercion (noPosition $ catToStr cat) -- This is weird: isCoercion is supposed to be applied to functions!!!!
     then fixAll top cats
     else (normCat cat, fixCoercion cat top) : fixAll top cats
 
 --A generic variable name for C-like languages.
+varName :: String -> String
 varName c = map toLower c ++ "_"
 
 --this makes var names a little cleaner.
-showNum n = if n == 0 then [] else show n
+showNum :: Int -> String
+showNum n = if n == 0 then "" else show n
 
 -- Makes the first letter a lowercase.
 firstLowerCase :: String -> String
